@@ -16,6 +16,7 @@ my %options = (
     log     => undef,
     default => undef,
     prefix  => undef,
+    oldcfg  => 0,
 );
 GetOptions( \%options, 
     'config|c=s', 'jcl|j=s', 'log|l=s', 
@@ -30,16 +31,18 @@ foreach my $opt (qw( config jcl log )) {
 }
 
 use vars qw( $VERSION $conf );
-$VERSION = '0.012';
+$VERSION = '0.016';
 
 eval { require $options{config} };
-print "Using '$options{config}' for defaults.\n" unless $@;
+$options{oldcfg} = 1, print "Using '$options{config}' for defaults.\n" 
+    unless $@;
 if ( $@ || $options{default} ) {
     my $df_key = $options{default} ? 'default' : 'prefix';
     my $df_config = "$options{ $df_key }_dfconfig";
     local $@;
     eval { require $df_config };
-    print "Using '$df_config' for more defaults.\n" unless $@;
+    $options{oldcfg} = 1, print "Using '$df_config' for more defaults.\n"
+        unless $@;
 } 
 
 =head1 NAME
@@ -354,6 +357,15 @@ Examples:$untarmsg",
         alt => [qw( Y n )],
         dft => 'y',
     },
+    killtime => {
+        msg => "Should this smoke be aborted on/after a specific time?
+\tuse HH:MM to specify a point in time (24 hour notation)
+\tuse +HH::MM to specify a duration
+\tleave empty to finish the smoke without aborting",
+        dft => "",
+        alt => [ ],
+        chk => '^(?:(?:\+\d+)|(?:(?:[0-1]?[0-9])|(?:2[0-3])):[0-5]?[0-9])|$',
+    },
     # Schedule stuff
     docron => {
         msg => 'Should the smoke be scheduled?',
@@ -481,6 +493,9 @@ BUILDDIR: {
     my $manifest  = File::Spec->catfile( $config{ $arg }, 'MANIFEST' );
     my $dot_patch = File::Spec->catfile( $config{ $arg }, '.patch' );
     if ( -e $manifest && -e $dot_patch ) {
+        $opt{use_old}->{dft} = $options{oldcfg} && 
+                               $conf->{ddir} eq $config{ddir}
+            ? 'y' : $opt{use_old}->{dft};
         my $use_old = lc prompt( 'use_old' );
         redo BUILDDIR unless $use_old eq 'y';
     }
@@ -613,7 +628,6 @@ See also L<Test::Smoke::Syncer>
 =cut
 
 $arg = $want_forest ? 'fsync' : 'sync_type';
-$conf->{sync_type} = $conf->{fsync} || 'rsync' unless $want_forest;
 $config{ $arg } = lc prompt( $arg );
 
 SYNCER: {
@@ -856,11 +870,12 @@ C<renice> will add a line in the shell-script that starts the smoke.
 
 =cut
 
-my( $umask, $renice );
 unless ( is_win32 ) {
-    $umask = prompt( 'umask' );
+    $arg = 'umask';
+    $config{ $arg } = prompt( $arg );
 
-    $renice = prompt( 'renice' );
+    $arg = 'renice';
+    $config{ $arg } = prompt( $arg );
 }
 
 =item v
@@ -882,6 +897,23 @@ is the same after syncing the source-tree.
 $arg = 'smartsmoke';
 $config{ $arg } = prompt_yn( $arg );
 
+=item killtime
+
+When C<< $Config{d_alarm} >> is found we can use C<alarm()> to abort 
+long running smokes. Leave this value empty to keep the old behaviour.
+
+    07:30 => F<mktest.pl> is aborted on 7:30 localtime
+   +23:45 => F<mktest.pl> is aborted after 23 hours and 45 minutes
+
+Thank you Jarkko for donating this suggestion.
+
+=cut
+
+if ( $Config{d_alarm} ) {
+    $arg = 'killtime';
+    $config{ $arg } = prompt( $arg );
+}
+
 =item schedule stuff
 
 =over 4
@@ -1020,12 +1052,22 @@ sub write_sh {
 # @{[ scalar localtime ]}
 #
 # $cronline
-@{[ renice( $renice ) ]}
+@{[ renice( $config{renice} ) ]}
 cd $cwd
+CFGNAME=$options{config}
+LOCKFILE=$options{prefix}.lck
+if test -f "\$LOCKFILE" && test -s "\$LOCKFILE" ; then
+    echo "We seem to be running (or remove \$LOCKFILE)" >& 2
+    exit 200
+fi
+echo "\$LOCKFILE" > "\$LOCKFILE"
+
 PATH=$cwd:$ENV{PATH}
 export PATH
-umask $umask
-$^X smokeperl.pl -c $options{config} \$\* > $options{log} 2>&1
+umask $config{umask}
+$^X smokeperl.pl -c "\$CFGNAME" \$\* > $options{log} 2>&1
+
+rm "\$LOCKFILE"
 EO_SH
     close MYSMOKESH or warn "Error writing '$jcl': $!";
 
@@ -1061,12 +1103,22 @@ $copycmd
 REM $atline
 
 set WD=$cwd\
+for \%\%D in ( \%WD\% ) do \%\%~dD
 cd "\%WD\%"
-set OLD_PATH=\%PATH\%
-set PATH=$cwd;\%PATH\%
-$^X smokeperl.pl -c $options{config} \%* > "\%WD\%\\$options{log}" 2>&1
-set PATH=\%OLD_PATH\%
-set WD=
+set CFGNAME=$options{config}
+set LOCKFILE=$options{prefix}.lck
+if NOT EXIST \%LOCKFILE\% goto START_SMOKE
+    FIND "\%LOCKFILE\%" \%LOCKFILE\% > NUL:
+    if NOT ERRORLEVEL 1 (echo We seem to be running [or remove \%LOCKFILE\%]>&2) && exit /B 200
+
+:START_SMOKE
+    echo \%LOCKFILE\% > \%LOCKFILE\%
+    set OLD_PATH=\%PATH\%
+    set PATH=$cwd;\%PATH\%
+    $^X smokeperl.pl -c "\%CFGNAME\%" \%* > "\%WD\%\\$options{log}" 2>&1
+    set PATH=\%OLD_PATH\%
+
+del \%LOCKFILE\%
 EO_BAT
     close MYSMOKEBAT or warn "Error writing '$jcl': $!";
 
@@ -1080,17 +1132,24 @@ sub prompt {
         @{ $opt{ $_[0] } }{qw( msg alt dft chk )};
 
     $df_val = $conf->{ $_[0] } if exists $conf->{ $_[0] };
-    return $df_val unless defined $message;
+    unless ( defined $message ) {
+        my $retval = defined $df_val ? $df_val : "undef";
+        (caller 1)[3] or print "Got [$retval]\n";
+        return $df_val;
+    }
     $message =~ s/\s+$//;
 
-    $chk ||= '.*';
-
-    my $default = defined $df_val ? $df_val : 'undef';
-    my $alts    = @$alt ? "<" . join( "|", @$alt ) . "> " : "";
-    print "\n$message\n";
-
     my %ok_val;
     %ok_val = map { (lc $_ => 1) } @$alt if @$alt;
+    $chk ||= '.*';
+
+    my $default = defined $df_val ? $df_val : 'undef';
+    if ( @$alt && defined $df_val ) {
+        $default = $alt->[0] unless exists $ok_val{ $df_val };
+    }
+    my $alts    = @$alt ? "<" . join( "|", @$alt ) . "> " : "";
+    print "\n$message\n";
+
     my $input;
     INPUT: {
         print "$alts\[$default] \$ ";
