@@ -216,15 +216,18 @@ for (<OUT>) {
     }
 }
 
-$Config{ccvers}	||= $Config{ccversion} || $Config{gccversion};
+$Config{ccvers}	||= $Config{gccversion} || $Config{ccversion};
+require Sys::Hostname;
+my $this_host = Sys::Hostname::hostname() || "nohost";
 
 print <<EOH;
 Automated smoke report for $Config{version} patch $rpt{patch} on $Config{osname} - $Config{osvers} ($Config{archname})
-          v$VERSION      using $Config{cc} version $Config{ccvers}
-O = OK
-F = Failure(s), extended report at the bottom
+($this_host) using $Config{cc} version $Config{ccvers}
+Report generated by Test::Smoke v$VERSION
+
+O = OK  F = Failure(s), extended report at the bottom
 ? = still running or test results not (yet) available
-Build failures during:       - = unknown
+Build failures during:       - = unknown or N/A
     c = Configure, m = make, t = make test-prep
 
 EOH
